/*
This dofile creates tables that reproduce results using labor productivity.
*/

************
* Preliminaries
************

set more off
clear all
est clear
program drop _all

local maindir = [REDACTED]
local instruments d_pernum d_persf d_lghwy					
local compvars rm_es rm_f hhe hhf						

use [REDACTED], replace

gen lprod = log(pqs/labor)

capt prog drop firstf								
prog firstf, eclass
eret post A B
eret scalar N = el(C,1,1)
eret scalar N_clust = el(D,1,1)
eret scalar r2 = el(E,1,1)
eret scalar J = el(F,1,1)
eret scalar Jp = el(G,1,1)
eret scalar FF = el(H,1,4)
eret scalar FFp = el(H,1,7)
end

************
* Estimation
************

foreach model in `compvars'{
eststo ols_`model' : qui reg lprod d_`model' yr*, cluster(cea)			
estadd scalar Nround = round(e(N),100)
estadd scalar Cround = round(e(N_clust),50)
qui ivregress gmm lprod yr* (d_`model' = `instruments'), cluster(cea) 		
mat A = e(b)									
mat B = e(V)
mat C = e(N)
mat D = e(N_clust)
mat E = e(r2)
qui estat overid
mat F = r(HansenJ)
mat G = r(p_HansenJ)
qui estat first
mat H = r(singleresults)
firstf
estadd scalar Nround = round(e(N),100)
estadd scalar Cround = round(e(N_clust),50)
eststo iv_`model'
}

estout ols* iv* using [REDACTED], replace cells(b(star fmt(%9.4f)) se(par fmt(%9.4f))) stats(Nround Cround r2 FF FFp J Jp, fmt(%9.0f %9.0f %9.4f %9.4f %9.4f %9.4f) label("Observations (rounded)" "Clusters (rounded)" "R2" " First-Stage F" "p Value" "Hausman J Statistic" "p Value") ) drop(_cons yr*) coll(,none) numbers style(tab)


capt prog drop pclean				
prog pclean, eclass
syntax anything(name=A)
mat coln `A' = rm_es rm_f hhe hhf
eret post `A'
end

capt prog drop mergemodels			
prog mergemodels, eclass
syntax namelist
tempname b V N tmp
foreach name of local namelist{
	qui est restore `name'
	mat `b' = nullmat(`b'), e(b)
	mat `b' = `b'[1,1..colsof(`b')-1]
	mat `tmp' = e(V)
	mat `tmp' = `tmp'[1..rowsof(`tmp')-1,1..colsof(`tmp')-1]
	capt confirm matrix `V'
	if _rc {
		mat `V' = `tmp'
	}
	else {
	mat `V' = ( `V' , J(rowsof(`V'),colsof(`tmp'),0) \ J(rowsof(`tmp'), colsof(`V'),0), `tmp' )
	mat `N' = nullmat(`N'), e(N)
	}
}
local names: colfullnames `b'
mat coln `V' = `names'
mat rown `V' = `names'
eret post `b' `V'
eret scalar N = el(`N',1,1)
end

preserve

collapse (max) yr* rm_es d_rm_es d_rm_f d_hhe d_hhf `instruments' (p10) q10 = lprod (p20) q20 = lprod (p30) q30 = lprod (p40) q40 = lprod (p50) q50 = lprod (p60) q60 = lprod (p70) q70 = lprod (p80) q80 = lprod (p90) q90 = lprod, by(cea year)			// This line generates the CEA-level quantiles of the tfpq distibution and collapses everything to that level

replace rm_es = exp(rm_es)  		
gen rm_es_sqr = rm_es^2		
gen rm_es_cub = rm_es^3			


foreach quant in 10 20 30 40 50 60 70 80 90{
	ren rm_es n_`quant'_1	
	ren rm_es_sqr n_`quant'_2
	ren rm_es_cub n_`quant'_3

	foreach model in `compvars' {
		eststo quant_`model'_`quant' : qui ivregress gmm q`quant' yr* n_`quant'_* (d_`model' = `instruments'), cluster(cea) 	
		qui test (n_`quant'_1 = 0) (n_`quant'_2 = 0) (n_`quant'_3 = 0)							
		mat pm_`quant' = nullmat(pm_`quant'), r(p)										
	}
	ren n_`quant'_1 rm_es		
	ren n_`quant'_2 rm_es_sqr
	ren n_`quant'_3 rm_es_cub
	pclean pm_`quant'														
	eststo p_`quant'														
	local list_`quant' quant_rm_es_`quant' quant_rm_f_`quant' quant_hhe_`quant' quant_hhf_`quant'					
	mergemodels `list_`quant''												
	estadd scalar Nround = round(e(N),100)
	eststo qm_`quant'														

}

estout qm* using [REDACTED], replace cells(b(fmt(%9.4f)) se(par fmt(%9.4f))) stats(Nround, fmt(%9.0f)) coll(,none) numbers drop(yr* n*) style(tab)
estout p* using [REDACTED], replace cells(b(fmt(%9.4f))) coll(,none) numbers style(tab)

restore

capt prog drop estclean
prog estclean, eclass
syntax anything(name=model)
tempname b V Nround Cround J Jp temp
local names b0 by1 `model' bp2 by3 bp1 by2 bds
qui estat overid
mat `b' = e(b)
mat `V' = e(V)
mat coleq `b' = ""
mat coleq `V' = ""
mat roweq `V' = ""
mat coln `b' = `names'
mat coln `V' = `names'
mat rown `V' = `names'
mat `Nround' = e(Nround)
mat `Cround' = e(Cround)
mat `J' = r(J)
mat `Jp' = r(J_p)
eret post `b' `V'
eret scalar Nround = el(`Nround',1,1)
eret scalar Cround = el(`Cround',1,1)
eret scalar J = el(`J',1,1)
eret scalar Jp = el(`Jp',1,1)
end

use [REDACTED], replace
gen lprod = log(pqs/labor)


************
* Estimation
************

local rescale = 100

foreach model in `compvars' {

	local sn2 (L5.lprod - {by3}*yr87 - {bs}*L5.d_`model')^2
	local sn1 (L5.lprod - {by3}*yr87 - {bs}*L5.d_`model')^1
	
	local n_series {bp2}*`sn2' + {bp1}*`sn1'
	local eps1 (lprod - {b0} - {by1}*yr92 - {bs}*d_`model' - (`n_series'))
	local eps2 (lprod - {b0} - {by2}*yr92- {bs}*d_`model' - {bds}*d_`model')
	local inst_l yr92 dl_pernum dl_persf dl_lghwy
	local inst_c yr92 d_pernum d_persf d_lghwy

	gmm (`eps1'*`rescale') (`eps2'*`rescale') (`eps1'*`sn2'*`rescale') (`eps1'* `sn1'*`rescale'), instruments(1:`inst_l') instruments(2:`inst_c') vce(cluster cea) winitial(identity) wmatrix(cluster cea) technique(dfp) from(b0 1.4 by1 0 by2 0 by3 0 bs .06 bds 0 bp2 0 bp1 0)
	estadd scalar Nround = round(e(N),100)
	estadd scalar Cround = round(e(N_clust),50)
	estimates save [REDACTED], replace
	disp("check1")
	estclean `model'
	disp("check2")
	eststo lprod_`model'
	disp("check3")
}

estout lprod_* using [REDACTED], replace cells(b(fmt(%9.4f)) se(par fmt(%9.4f))) stats(Nround Cround J Jp, fmt(%9.0f %9.0f %9.4f %9.4f)) label coll(,none) numbers keep(`compvars' bds) style(tab)

